@@ -23,7 +23,11 @@
 
 #include "ephy-embed.h"
 
+#ifdef HAVE_WEBKIT2
+#include <webkit2/webkit2.h>
+#else
 #include <libsoup/soup.h>
+#endif
 
 #if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
 #error "Only <epiphany/epiphany.h> can be included directly."
@@ -34,11 +38,16 @@
 
 G_BEGIN_DECLS
 
-void ephy_embed_prefs_init                  (void);
-void ephy_embed_prefs_shutdown              (void);
-void ephy_embed_prefs_add_embed             (EphyEmbed     *embed);
-void ephy_embed_prefs_set_cookie_jar_policy (SoupCookieJar *jar,
-                                             const char    *gconf_policy);
+void ephy_embed_prefs_init                     (void);
+void ephy_embed_prefs_shutdown                 (void);
+void ephy_embed_prefs_add_embed                (EphyEmbed           *embed);
+#ifdef HAVE_WEBKIT2
+void ephy_embed_prefs_set_cookie_accept_policy (WebKitCookieManager *cookie_manager,
+                                                const char          *settings_policy);
+#else
+void ephy_embed_prefs_set_cookie_jar_policy    (SoupCookieJar       *jar,
+                                                const char          *gconf_policy);
+#endif
 
 G_END_DECLS
 
