@@ -22,6 +22,7 @@
 #ifndef EPHY_EMBED_H
 #define EPHY_EMBED_H
 
+#include "ephy-overview.h"
 #include "ephy-web-view.h"
 
 #include <gtk/gtk.h>
@@ -61,6 +62,10 @@ void         ephy_embed_auto_download_url        (EphyEmbed  *embed,
                                                   const char *url);
 void         ephy_embed_entering_fullscreen      (EphyEmbed *embed);
 void         ephy_embed_leaving_fullscreen       (EphyEmbed *embed);
+void         ephy_embed_set_overview_mode        (EphyEmbed *embed,
+                                                  gboolean   overview_mode);
+gboolean     ephy_embed_get_overview_mode        (EphyEmbed *embed);
+EphyOverview*ephy_embed_get_overview             (EphyEmbed *embed);
 
 G_END_DECLS
 
