@@ -37,6 +37,7 @@
 #include "ephy-history-service.h"
 #include "ephy-profile-utils.h"
 #include "ephy-settings.h"
+#include "ephy-sqlite-connection.h"
 #include "ephy-web-app-utils.h"
 #ifdef ENABLE_NSS
 #include "ephy-nss-glue.h"
@@ -51,6 +52,8 @@
 #include <sys/types.h>
 
 static int do_step_n = -1;
+static int version = -1;
+static char *profile_dir = NULL;
 
 /*
  * What to do to add new migration steps:
@@ -60,6 +63,12 @@ static int do_step_n = -1;
 
 typedef void (*EphyProfileMigrator) (void);
 
+static gboolean
+profile_dir_exists ()
+{
+  return g_file_test (ephy_dot_dir (), G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR);
+}
+
 static void
 migrate_cookies ()
 {
@@ -763,6 +772,49 @@ migrate_web_app_links ()
   ephy_web_application_free_application_list (apps);
 }
 
+static void
+migrate_new_urls_table ()
+{
+  EphySQLiteConnection *history_database;
+  char *filename;
+  GError *error = NULL;
+
+  filename = g_build_filename (ephy_dot_dir (), "ephy-history.db", NULL);
+  history_database = ephy_sqlite_connection_new ();
+  ephy_sqlite_connection_open (history_database, filename, &error);
+
+  if (error) {
+    g_warning ("Failed to open history database: %s\n", error->message);
+    g_error_free (error);
+    g_free (filename);
+    return;
+  }
+
+  ephy_sqlite_connection_execute (history_database,
+                                  "ALTER TABLE urls "
+                                  "ADD COLUMN thumbnail_update_time INTEGER DEFAULT 0",
+                                  &error);
+  if (error) {
+    g_warning ("Failed to add new column to table in history backend: %s\n",
+               error->message);
+    g_error_free (error);
+    error = NULL;
+  }
+  ephy_sqlite_connection_execute (history_database,
+                                  "ALTER TABLE urls "
+                                  "ADD COLUMN hidden_from_overview INTEGER DEFAULT 0",
+                                  &error);
+  if (error) {
+    g_warning ("Failed to add new column to table in history backend: %s\n",
+               error->message);
+    g_error_free (error);
+    error = NULL;
+  }
+
+  g_object_unref (history_database);
+  g_free (filename);
+}
+
 const EphyProfileMigrator migrators[] = {
   migrate_cookies,
   migrate_passwords,
@@ -774,7 +826,8 @@ const EphyProfileMigrator migrators[] = {
   migrate_passwords2,
   migrate_history,
   migrate_tabs_visibility,
-  migrate_web_app_links
+  migrate_web_app_links,
+  migrate_new_urls_table,
 };
 
 static gboolean
@@ -787,6 +840,12 @@ ephy_migrator ()
    * very beginning. */
   migrate_profile_gnome2_to_xdg ();
 
+  /* If after this point there's no profile dir, there's no point in
+   * running anything because Epiphany has never run in this sytem, so
+   * exit here. */
+  if (!profile_dir_exists ())
+    return TRUE;
+
   if (do_step_n != -1) {
     if (do_step_n >= EPHY_PROFILE_MIGRATION_VERSION)
       return FALSE;
@@ -829,6 +888,10 @@ static const GOptionEntry option_entries[] =
 {
   { "do-step", 'd', 0, G_OPTION_ARG_INT, &do_step_n,
     N_("Executes only the n-th migration step"), NULL },
+  { "version", 'v', 0, G_OPTION_ARG_INT, &version,
+    N_("Specifies the required version for the migrator"), NULL },
+  { "profile-dir", 'p', 0, G_OPTION_ARG_FILENAME, &profile_dir,
+    N_("Specifies the profile where the migrator should run"), NULL },
   { NULL }
 };
 
@@ -838,8 +901,7 @@ main (int argc, char *argv[])
   GOptionContext *option_context;
   GOptionGroup *option_group;
   GError *error = NULL;
-
-  g_type_init ();
+  EphyFileHelpersFlags file_helpers_flags = EPHY_FILE_HELPERS_NONE;
 
   option_group = g_option_group_new ("ephy-profile-migrator",
                                      N_("Epiphany profile migrator"),
@@ -862,9 +924,19 @@ main (int argc, char *argv[])
         
   g_option_context_free (option_context);
 
+  if (version != -1 && version != EPHY_PROFILE_MIGRATION_VERSION) {
+    g_print ("Version mismatch, version %d requested but our version is %d\n", version, EPHY_PROFILE_MIGRATION_VERSION);
+    
+    return 1;
+  }
+
   ephy_debug_init ();
 
-  if (!ephy_file_helpers_init (NULL, EPHY_FILE_HELPERS_NONE, NULL)) {
+  if (profile_dir != NULL)
+    file_helpers_flags = EPHY_FILE_HELPERS_PRIVATE_PROFILE |
+      EPHY_FILE_HELPERS_KEEP_DIR;
+
+  if (!ephy_file_helpers_init (profile_dir, file_helpers_flags, NULL)) {
     LOG ("Something wrong happened with ephy_file_helpers_init()");
     return -1;
   }
