@@ -291,11 +291,7 @@ get_target_window (EphyBookmarksEditor *editor)
 	}
 	else
 	{
-		EphySession *session;
-
-		session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
-
-		return GTK_WIDGET (ephy_session_get_active_window (session));
+                return GTK_WIDGET (ephy_shell_get_active_window (ephy_shell));
 	}
 }
 
@@ -1499,9 +1495,6 @@ icon_loaded_cb (WebKitFaviconDatabase *database, GAsyncResult *result, GtkTreeRo
 static void
 provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 {
-#ifdef HAVE_WEBKIT2
-        /* TODO: Favicons */
-#else
 	const char *page_location;
 	GdkPixbuf *favicon = NULL;
 
@@ -1510,6 +1503,9 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 
 	LOG ("Get favicon for %s", page_location ? page_location : "None");
 
+#ifdef HAVE_WEBKIT2
+        /* TODO: Favicons */
+#else
 	if (page_location)
         {
 		WebKitFaviconDatabase *database = webkit_get_favicon_database ();
@@ -1537,10 +1533,9 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 			}
 		}
         }
-
+#endif
 	g_value_init (value, GDK_TYPE_PIXBUF);
 	g_value_take_object (value, favicon);
-#endif
 }
 
 static void
