@@ -42,17 +42,12 @@
 #include <libxml/xmlversion.h>
 #include <string.h>
 
-#ifdef ENABLE_INTROSPECTION
-#include <girepository.h>
-#endif
-
 static GQuark startup_error_quark = 0;
 #define STARTUP_ERROR_QUARK (startup_error_quark)
 
 static gboolean open_in_new_tab = FALSE;
 static gboolean open_in_new_window = FALSE;
 static gboolean open_as_bookmarks_editor = FALSE;
-/*static gboolean reload_plugins = FALSE;*/
 
 static char *session_filename = NULL;
 static char *bookmark_url = NULL;
@@ -331,10 +326,6 @@ main (int argc,
 
   g_option_context_set_main_group (option_context, option_group);
 
-#ifdef ENABLE_INTROSPECTION
-  g_option_context_add_group (option_context, g_irepository_get_option_group ());
-#endif
-
   g_option_context_add_group (option_context, gtk_get_option_group (TRUE));
 
 #ifdef GNOME_ENABLE_DEBUG
@@ -389,9 +380,15 @@ main (int argc,
   /* Work-around Flash Player crash */
   g_setenv ("XLIB_SKIP_ARGB_VISUALS", "1", FALSE);
 
-  /* TODO: we want to migrate each WebApp profile too. */
-  if (!private_instance && !application_mode)
-    ephy_profile_utils_do_migration (-1, FALSE);
+  /* Run the migration in all cases, except when running a private
+     instance without a given profile directory. */
+  if (!(private_instance && profile_directory == FALSE)) {
+    /* If the migration fails we don't really want to continue. */
+    if (!ephy_profile_utils_do_migration ((const char *)profile_directory, -1, FALSE)) {
+      g_print ("Failed to run the migrator process, Web will now abort.");
+      exit (1);
+    }
+  }
 
   /* Start our services */
   flags = EPHY_FILE_HELPERS_ENSURE_EXISTS;
@@ -399,7 +396,7 @@ main (int argc,
   if (private_instance || application_mode)
     flags |= EPHY_FILE_HELPERS_PRIVATE_PROFILE;
   if (keep_temp_directory || profile_directory)
-    flags |= EPHY_FILE_HELPERS_KEEP_TEMP_DIR;
+    flags |= EPHY_FILE_HELPERS_KEEP_DIR;
 
   if (!ephy_file_helpers_init (profile_directory, flags,
                                &error)) {
