@@ -23,13 +23,13 @@
 #include "config.h"
 #include "ephy-shell.h"
 
+#include "ephy-adblock-manager.h"
 #include "ephy-bookmarks-editor.h"
 #include "ephy-bookmarks-import.h"
 #include "ephy-debug.h"
 #include "ephy-embed-container.h"
 #include "ephy-embed-single.h"
 #include "ephy-embed-utils.h"
-#include "ephy-extensions-manager.h"
 #include "ephy-file-helpers.h"
 #include "ephy-gui.h"
 #include "ephy-history-window.h"
@@ -53,9 +53,9 @@
 
 struct _EphyShellPrivate {
   EphySession *session;
+  GList *windows;
   GObject *lockdown;
   EphyBookmarks *bookmarks;
-  EphyExtensionsManager *extensions_manager;
   GNetworkMonitor *network_monitor;
   GtkWidget *bme;
   GtkWidget *history_window;
@@ -205,11 +205,9 @@ show_about (GSimpleAction *action,
             GVariant *parameter,
             gpointer user_data)
 {
-  EphySession *session;
   EphyWindow *window;
 
-  session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
-  window = ephy_session_get_active_window (session);
+  window = ephy_shell_get_active_window (ephy_shell);
 
   window_cmd_help_about (NULL, GTK_WIDGET (window));
 }
@@ -427,6 +425,61 @@ ephy_shell_before_emit (GApplication *application,
                                                               platform_data);
 }
 
+static gboolean
+window_focus_in_event_cb (EphyWindow *window,
+                          GdkEventFocus *event,
+                          EphyShell *shell)
+{
+  LOG ("focus-in-event for window %p", window);
+
+  g_return_val_if_fail (g_list_find (shell->priv->windows, window) != NULL, FALSE);
+
+  /* move the active window to the front of the list */
+  shell->priv->windows = g_list_remove (shell->priv->windows, window);
+  shell->priv->windows = g_list_prepend (shell->priv->windows, window);
+
+  return GDK_EVENT_PROPAGATE;
+}
+
+static void
+ephy_shell_window_added (GtkApplication *application,
+                         GtkWindow *window)
+{
+  EphyShell *shell = EPHY_SHELL (application);
+
+  if (EPHY_IS_WINDOW (window)) {
+    shell->priv->windows = g_list_append (shell->priv->windows, window);
+    g_signal_connect (window, "focus-in-event",
+                      G_CALLBACK (window_focus_in_event_cb),
+                      shell);
+  }
+
+  GTK_APPLICATION_CLASS (ephy_shell_parent_class)->window_added (application, window);
+}
+
+static void
+ephy_shell_window_removed (GtkApplication *application,
+                           GtkWindow *window)
+{
+  EphyShell *shell = EPHY_SHELL (application);
+
+  if (EPHY_IS_WINDOW (window))
+    shell->priv->windows = g_list_remove (shell->priv->windows, window);
+
+  GTK_APPLICATION_CLASS (ephy_shell_parent_class)->window_removed (application, window);
+}
+
+static GObject *
+ephy_shell_get_lockdown (EphyShell *shell)
+{
+  g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
+
+  if (shell->priv->lockdown == NULL)
+    shell->priv->lockdown = g_object_new (EPHY_TYPE_LOCKDOWN, NULL);
+
+  return G_OBJECT (shell->priv->session);
+}
+
 static void
 ephy_shell_constructed (GObject *object)
 {
@@ -438,6 +491,12 @@ ephy_shell_constructed (GObject *object)
     g_application_set_flags (G_APPLICATION (object), flags);
   }
 
+  /* FIXME: not sure if this is the best place to put this stuff. */
+  ephy_shell_get_lockdown (EPHY_SHELL (object));
+  
+  if (ephy_embed_shell_get_mode (EPHY_EMBED_SHELL (object)) != EPHY_EMBED_SHELL_MODE_TEST)
+    ephy_embed_shell_get_adblock_manager (EPHY_EMBED_SHELL (object));
+
   if (G_OBJECT_CLASS (ephy_shell_parent_class)->constructed)
     G_OBJECT_CLASS (ephy_shell_parent_class)->constructed (object);
 }
@@ -447,6 +506,7 @@ ephy_shell_class_init (EphyShellClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GApplicationClass *application_class = G_APPLICATION_CLASS (klass);
+  GtkApplicationClass *gtk_application_class = GTK_APPLICATION_CLASS (klass);
   EphyEmbedShellClass *embed_shell_class = EPHY_EMBED_SHELL_CLASS (klass);
 
   object_class->dispose = ephy_shell_dispose;
@@ -458,6 +518,9 @@ ephy_shell_class_init (EphyShellClass *klass)
   application_class->before_emit = ephy_shell_before_emit;
   application_class->add_platform_data = ephy_shell_add_platform_data;
 
+  gtk_application_class->window_added = ephy_shell_window_added;
+  gtk_application_class->window_removed = ephy_shell_window_removed;
+
   embed_shell_class->get_embed_single = impl_get_embed_single;
 
   g_type_class_add_private (object_class, sizeof(EphyShellPrivate));
@@ -528,7 +591,6 @@ download_started_cb (WebKitWebContext *web_context,
                      EphyShell *shell)
 {
   EphyDownload *ed;
-  EphySession *session;
   EphyWindow *window;
 
   /* Is download locked down? */
@@ -538,8 +600,7 @@ download_started_cb (WebKitWebContext *web_context,
     return;
   }
 
-  session = EPHY_SESSION (ephy_shell_get_session (shell));
-  window = ephy_session_get_active_window (session);
+  window = ephy_shell_get_active_window (shell);
 
   ed = ephy_download_new_for_download (download);
   ephy_download_set_window (ed, GTK_WIDGET (window));
@@ -569,64 +630,23 @@ ephy_shell_init (EphyShell *shell)
 static void
 ephy_shell_dispose (GObject *object)
 {
-  EphyShell *shell = EPHY_SHELL (object);
-  EphyShellPrivate *priv = shell->priv;
+  EphyShellPrivate *priv = EPHY_SHELL (object)->priv;
 
   LOG ("EphyShell disposing");
 
-  if (shell->priv->extensions_manager != NULL) {
-    LOG ("Unref extension manager");
-    /* this will unload the extensions */
-    g_object_unref (priv->extensions_manager);
-    priv->extensions_manager = NULL;
-  }
+  g_clear_object (&priv->session);
+  g_clear_object (&priv->lockdown);
+  g_clear_pointer (&priv->bme, gtk_widget_destroy);
+  g_clear_pointer (&priv->history_window, gtk_widget_destroy);
+  g_clear_object (&priv->pdm_dialog);
+  g_clear_object (&priv->prefs_dialog);
+  g_clear_object (&priv->bookmarks);
+  g_clear_object (&priv->network_monitor);
 
-  if (priv->session != NULL) {
-    LOG ("Unref session manager");
-    g_object_unref (priv->session);
-    priv->session = NULL;
-  }
-
-  if (priv->lockdown != NULL) {
-    LOG ("Unref lockdown controller");
-    g_object_unref (priv->lockdown);
-    priv->lockdown = NULL;
-  }
-
-  if (priv->bme != NULL) {
-    LOG ("Unref Bookmarks Editor");
-    gtk_widget_destroy (GTK_WIDGET (priv->bme));
-    priv->bme = NULL;
-  }
-
-  if (priv->history_window != NULL) {
-    LOG ("Unref History Window");
-    gtk_widget_destroy (GTK_WIDGET (priv->history_window));
-    priv->history_window = NULL;
-  }
-
-  if (priv->pdm_dialog != NULL) {
-    LOG ("Unref PDM Dialog");
-    g_object_unref (priv->pdm_dialog);
-    priv->pdm_dialog = NULL;
-  }
-
-  if (priv->prefs_dialog != NULL) {
-    LOG ("Unref prefs dialog");
-    g_object_unref (priv->prefs_dialog);
-    priv->prefs_dialog = NULL;
-  }
-
-  if (priv->bookmarks != NULL) {
-    LOG ("Unref bookmarks");
-    g_object_unref (priv->bookmarks);
-    priv->bookmarks = NULL;
-  }
-
-  if (priv->network_monitor != NULL) {
-    LOG ("Unref net monitor ");
-    g_object_unref (priv->network_monitor);
-    priv->network_monitor = NULL;
+  if (priv->windows != NULL) {
+    LOG ("Free browser window list");
+    g_list_free (priv->windows);
+    priv->windows = NULL;
   }
 
   G_OBJECT_CLASS (ephy_shell_parent_class)->dispose (object);
@@ -853,44 +873,9 @@ ephy_shell_get_session (EphyShell *shell)
 {
   g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
 
-  if (shell->priv->session == NULL) {
-    EphyExtensionsManager *manager;
-
+  if (shell->priv->session == NULL)
     shell->priv->session = g_object_new (EPHY_TYPE_SESSION, NULL);
 
-    manager = EPHY_EXTENSIONS_MANAGER
-      (ephy_shell_get_extensions_manager (shell));
-    ephy_extensions_manager_register (manager,
-                                      G_OBJECT (shell->priv->session));
-  }
-
-  return G_OBJECT (shell->priv->session);
-}
-
-/**
- * ephy_shell_get_lockdown:
- * @shell: the #EphyShell
- *
- * Returns the lockdown controller.
- *
- * Return value: the lockdown controller
- **/
-static GObject *
-ephy_shell_get_lockdown (EphyShell *shell)
-{
-  g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
-
-  if (shell->priv->lockdown == NULL) {
-    EphyExtensionsManager *manager;
-
-    shell->priv->lockdown = g_object_new (EPHY_TYPE_LOCKDOWN, NULL);
-
-    manager = EPHY_EXTENSIONS_MANAGER
-      (ephy_shell_get_extensions_manager (shell));
-    ephy_extensions_manager_register (manager,
-                                      G_OBJECT (shell->priv->lockdown));
-  }
-
   return G_OBJECT (shell->priv->session);
 }
 
@@ -909,31 +894,6 @@ ephy_shell_get_bookmarks (EphyShell *shell)
   return shell->priv->bookmarks;
 }
 
-/**
- * ephy_shell_get_extensions_manager:
- *
- * Return value: (transfer none):
- **/
-GObject *
-ephy_shell_get_extensions_manager (EphyShell *es)
-{
-  g_return_val_if_fail (EPHY_IS_SHELL (es), NULL);
-
-  if (es->priv->extensions_manager == NULL) {
-    /* Instantiate extensions manager */
-    es->priv->extensions_manager =
-      g_object_new (EPHY_TYPE_EXTENSIONS_MANAGER, NULL);
-
-    ephy_extensions_manager_startup (es->priv->extensions_manager);
-
-    /* FIXME */
-    ephy_shell_get_lockdown (es);
-    ephy_embed_shell_get_adblock_manager (embed_shell);
-  }
-
-  return G_OBJECT (es->priv->extensions_manager);
-}
-
 /**
  * ephy_shell_get_net_monitor:
  *
@@ -1064,3 +1024,61 @@ ephy_shell_set_startup_context (EphyShell *shell,
 
   shell->priv->startup_context = ctx;
 }
+
+GList *
+ephy_shell_get_windows (EphyShell *shell)
+{
+  g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
+
+  return g_list_copy (shell->priv->windows);
+}
+
+guint
+ephy_shell_get_n_windows (EphyShell *shell)
+{
+  g_return_val_if_fail (EPHY_IS_SHELL (shell), 0);
+
+  return g_list_length (shell->priv->windows);
+}
+
+EphyWindow *
+ephy_shell_get_active_window (EphyShell *shell)
+{
+  GList *l;
+
+  g_return_val_if_fail (EPHY_IS_SHELL (shell), NULL);
+
+  for (l = shell->priv->windows; l != NULL; l = l->next) {
+    EphyEmbedContainer *window = EPHY_EMBED_CONTAINER (l->data);
+
+    if (!ephy_embed_container_get_is_popup (window))
+      return EPHY_WINDOW (window);
+  }
+
+  return NULL;
+}
+
+gboolean
+ephy_shell_close_all_windows (EphyShell *shell)
+{
+  GList *windows;
+  gboolean retval = TRUE;
+
+  g_return_val_if_fail (EPHY_IS_SHELL (shell), FALSE);
+
+  ephy_session_close (EPHY_SESSION (ephy_shell_get_session (shell)));
+
+  windows = shell->priv->windows;
+  while (windows) {
+    EphyWindow *window = EPHY_WINDOW (windows->data);
+
+    windows = windows->next;
+
+    if (ephy_window_close (window))
+      gtk_widget_destroy (GTK_WIDGET (window));
+    else
+      retval = FALSE;
+  }
+
+  return retval;
+}
