@@ -167,8 +167,6 @@ GObject        *ephy_shell_get_net_monitor              (EphyShell *shell);
 
 EphyBookmarks  *ephy_shell_get_bookmarks                (EphyShell *shell);
 
-GObject        *ephy_shell_get_extensions_manager       (EphyShell *shell);
-
 GtkWidget      *ephy_shell_get_bookmarks_editor         (EphyShell *shell);
 
 GtkWidget      *ephy_shell_get_history_window           (EphyShell *shell);
@@ -177,6 +175,14 @@ GObject        *ephy_shell_get_pdm_dialog               (EphyShell *shell);
 
 GObject        *ephy_shell_get_prefs_dialog             (EphyShell *shell);
 
+GList          *ephy_shell_get_windows                  (EphyShell *shell);
+
+guint           ephy_shell_get_n_windows                (EphyShell *shell);
+
+EphyWindow     *ephy_shell_get_active_window            (EphyShell *shell);
+
+gboolean        ephy_shell_close_all_windows            (EphyShell *shell);
+
 G_END_DECLS
 
 #endif
