@@ -24,7 +24,7 @@
 
 /* From GTK_CHECK_VERSION */
 #define EPHY_MAJOR_VERSION (3)
-#define EPHY_MINOR_VERSION (6)
+#define EPHY_MINOR_VERSION (7)
 #define EPHY_MICRO_VERSION (1)
 
 #define EPHY_CHECK_VERSION(major, minor, micro)\
@@ -47,11 +47,8 @@
 #include <epiphany/ephy-embed-single.h>
 #include <epiphany/ephy-embed-type-builtins.h>
 #include <epiphany/ephy-embed-utils.h>
-#include <epiphany/ephy-extension.h>
-#include <epiphany/ephy-extensions-manager.h>
 #include <epiphany/ephy-lib-type-builtins.h>
 #include <epiphany/ephy-link.h>
-#include <epiphany/ephy-loader.h>
 #include <epiphany/ephy-node-db.h>
 #include <epiphany/ephy-node.h>
 #include <epiphany/ephy-notebook.h>
