@@ -985,12 +985,12 @@ static const PrefsDialogPreference preferences[] =
 	{ "popups_allow_checkbutton", "active",
 	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_POPUPS,
 	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
+	{ "adblock_allow_checkbutton", "active",
+	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_ADBLOCK,
+	  G_SETTINGS_BIND_INVERT_BOOLEAN, NULL, NULL },
 	{ "enable_plugins_checkbutton", "active",
 	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_PLUGINS,
 	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
-	{ "enable_javascript_checkbutton", "active",
-	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_JAVASCRIPT,
-	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
 	{ "css_checkbox", "active",
 	  EPHY_PREFS_WEB_SCHEMA, EPHY_PREFS_WEB_ENABLE_USER_CSS,
 	  G_SETTINGS_BIND_DEFAULT, NULL, NULL },
