@@ -165,7 +165,7 @@ main (int argc, char *argv[])
 
   ephy_debug_init ();
   ephy_embed_prefs_init ();
-  _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_PRIVATE);
+  _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_TEST);
 
   if (!ephy_file_helpers_init (NULL,
                                EPHY_FILE_HELPERS_PRIVATE_PROFILE | EPHY_FILE_HELPERS_ENSURE_EXISTS,
