@@ -344,7 +344,7 @@ main (int argc, char *argv[])
     return -1;
   }
 
-  _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_PRIVATE);
+  _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_TEST);
   g_application_register (G_APPLICATION (ephy_shell), NULL, NULL);
 
   g_test_add_func ("/src/ephy-shell/basic_embeds",
